import XCTest
import Foundation
#if canImport(FoundationNetworking)
import FoundationNetworking
#endif
import Appwrite
import AsyncHTTPClient
import NIO

class Tests: XCTestCase {

    override func setUp() {
        super.setUp()
        print("Test Started")
    }

    override func tearDown() {
        super.tearDown()
    }

    func test() async throws {
        let client = Client()
            .setEndpointRealtime("ws://demo.appwrite.io/v1")
            .setProject("console")
            .addHeader(key: "Origin", value: "http://localhost")
            .setSelfSigned()

        let foo = Foo(client, "string")
        let bar = Bar(client)
        let general = General(client)
        let realtime = Realtime(client)
        var realtimeResponse = "Realtime failed!"

        let expectation = XCTestExpectation(description: "realtime server")    
        
        realtime.subscribe(channels: ["tests"]) { message in
            realtimeResponse = message.payload!["response"] as! String
            expectation.fulfill()
        }
        
        var mock: Mock

        // Foo Tests
        mock = try await foo.get(y: 123, z: ["string in array"])
        print(mock.result)

        mock = try await foo.post(y: 123, z: ["string in array"])
        print(mock.result)

        mock = try await foo.put(y: 123, z: ["string in array"])
        print(mock.result)

        mock = try await foo.patch(y: 123, z: ["string in array"])
        print(mock.result)

        mock = try await foo.delete(y: 123, z: ["string in array"])
        print(mock.result)


        // Bar Tests
        mock = try await bar.get(xrequired: "string", xdefault: 123, z: ["string in array"])
        print(mock.result)

        mock = try await bar.post(xrequired: "string", xdefault: 123, z: ["string in array"])
        print(mock.result)

        mock = try await bar.put(xrequired: "string", xdefault: 123, z: ["string in array"])
        print(mock.result)

        mock = try await bar.patch(xrequired: "string", xdefault: 123, z: ["string in array"])
        print(mock.result)

        mock = try await bar.delete(xrequired: "string", xdefault: 123, z: ["string in array"])
        print(mock.result)


        // General Tests
        let result = try await general.redirect()
        print((result as! [String: Any])["result"] as! String)

        var url = URL(fileURLWithPath: "\(FileManager.default.currentDirectoryPath)/../../resources/file.png")
        var buffer = ByteBuffer(data: try! Data(contentsOf: url))
        var file = File(name: "file.png", buffer: buffer)
        mock = try await general.upload(x: "string", y: 123, z: ["string in array"], file: file, onProgress: nil)
        print(mock.result)

        url = URL(fileURLWithPath: "\(FileManager.default.currentDirectoryPath)/../../resources/large_file.mp4")
        buffer = ByteBuffer(data: try! Data(contentsOf: url))
        file = File(name: "large_file.mp4", buffer: buffer)
		mock = try await general.upload(x: "string", y: 123, z: ["string in array"], file: file, onProgress: nil)
		print(mock.result)

        do {
            try await general.error400()
        } catch let error as AppwriteError {
            print(error.message)
        }

        do {
            try await general.error500()
        } catch {
            print(error.localizedDescription)
        }

        do {
            try await general.error502()
        } catch {
            print(String(describing: error))
        }

        wait(for: [expectation], timeout: 10.0)
        print(realtimeResponse)

        mock = try await general.setCookie()
        print(mock.result)

        mock = try await general.getCookie()
        print(mock.result)

        try! await general.empty()
    }
}
